/* pdid.c -- daemon to monitor a parallel port device interface board
 *          as described in the January 1993 issue of Computer Craft,
 *          "Recycling Old Computers" by Raymond H. Green.
 * 
 * Copyright 1993 Thomas Insel.
 */

#include <fcntl.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#include <sys/timeb.h>

#define LPT_PORT 888
#define CONFIG_FILE "temp.conf"

char	pin_command[100][21];	
char	pin_bouncy[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};
char	pin[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};
int	hand[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};
long	pins_lasttrigger[21];
	/* there are 25 pins, but we don't use any outside of 1-20 */

int can_process(pin)
int pin;
{
	long current_time, difference;
		
	current_time = time(0);
	difference = current_time - pins_lasttrigger[pin];

	pins_lasttrigger[pin] = current_time;
	return ( difference > 5);

	}

read_config(prog_name)
char	*prog_name;
{
	FILE	*fred;
	char	line_buffer[255], buff1[5], buff2[100], buff3[10];
	int	pin_temp, i;

	for (i=1;i<21;i++) pin_command[i][0]='-';

	if ( (fred=fopen(CONFIG_FILE,"r")) == NULL ) {
		fprintf(stderr,"%s: cannot open CONFIG_FILE.\n",prog_name);
		exit(-1);
		}

	while(1) {
		fgets(line_buffer,254,fred);
		if (feof(fred)) break;
		strcpy(buff3,"-");
		sscanf(line_buffer,"%s %s %s",buff1,buff2,buff3);
		pin_temp=atoi(buff1);
		if ( pin_temp>0 & pin_temp<21 ) {
			strcpy(pin_command[pin_temp],buff2);
			if (buff3[0] == 'b') pin_bouncy[pin_temp]=1;
			}
		}

	fclose(fred);

	}

readpins(int filedes) {
		unsigned char biff[2];

		lseek(filedes,LPT_PORT+1,0);
		read(filedes, &biff[0], 2);

		pin[17] = ( (biff[1] & 8) == 0);	
		pin[16] = ( (biff[1] & 4) == 4);	
	 	pin[15] = ( (biff[0] & 8) == 8);
		pin[14] = ( (biff[1] & 2) == 0);	
	 	pin[13] = ( (biff[0] & 16) == 16);
	 	pin[12] = ( (biff[0] & 32) == 32);
	 	pin[11] = ( (biff[0] & 128) == 0);
	 	pin[10] = ( (biff[0] & 64) == 64);
		pin[1]  = ( (biff[1] & 1) == 0);	
	}

handle_pin(i)
int i;
{
	if ( pin[i]==0 && hand[i]==0 && strlen(pin_command[i])>1 ) {

		if (pin_bouncy[i]==0 || can_process(i)) {
			if ((hand[i] = fork()) <0) { /*error*/
				hand[i]=0;
				}
			else if (hand[i] == 0) { /*child*/
				printf("%d\n",i);
				system(pin_command[i]);
				exit(1);
				}
			}

		}

	if (hand[i] != 0) {
		if ( waitpid(hand[i],0,WNOHANG) != 0 ) hand[i]=0;
		}	

	}

main(argc, argv) 
int 	argc;
char	*argv[];
{
	char	biff;
	int	port_fd;
	int 	i, running=1;

	for (i=1;i<21;i++) pins_lasttrigger[i]=0;

	if ( (port_fd = open("/dev/port",O_RDWR)) < 0 ) {
		printf("%s: cannot open /dev/port.\n",argv[0]);
		exit(-1);
		}

	/* Initialize pin 16 to high 
         *  -- i should initialize all I/O pins to a know state.
         */
	lseek(port_fd,LPT_PORT+2,0);   
	biff=4;
	write(port_fd,&biff,1);

	read_config(argv[0]);

	while (running) {
		usleep(200); /*don't eat too much processor time*/

		readpins(port_fd);

		for (i=10;i<=17;i++) handle_pin(i);
		handle_pin(1);

		} /*while*/

	close(port_fd);	
	}
